'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable valid-jsdoc, jsdoc/require-description-complete-sentence, jsdoc/require-param, jsdoc/check-param-names, filenames/match-exported */

/* eslint-disable react/no-set-state, react/no-did-mount-set-state, react/sort-prop-types */
var Carousel = function (_Component) {
  _inherits(Carousel, _Component);

  function Carousel(props) {
    _classCallCheck(this, Carousel);

    var _this = _possibleConstructorReturn(this, (Carousel.__proto__ || Object.getPrototypeOf(Carousel)).call(this, props));

    _this.getIndexToScrollTo = function (_ref) {
      var totalItems = _ref.totalItems,
          firstVisibleIndex = _ref.firstVisibleIndex,
          direction = _ref.direction,
          visibleItemsCount = _ref.visibleItemsCount,
          scrollStepDistance = _ref.scrollStepDistance;

      var index = void 0;

      var itemsBeyondVisible = totalItems - firstVisibleIndex - visibleItemsCount;
      var itemsBehindVisible = firstVisibleIndex;

      if (direction === 'next') {
        index = firstVisibleIndex + (itemsBeyondVisible > scrollStepDistance ? scrollStepDistance : itemsBeyondVisible);
      } else if (direction === 'previous') {
        index = firstVisibleIndex - (itemsBehindVisible > scrollStepDistance ? scrollStepDistance : itemsBehindVisible);
      }

      return index;
    };

    _this.isNextButtonVisible = function (firstVisibleIndex, totalItems, visibleItemsCount) {
      return firstVisibleIndex < totalItems - visibleItemsCount;
    };

    _this.isPrevButtonVisible = function (firstVisibleIndex) {
      return firstVisibleIndex > 0;
    };

    _this.getVisibleItemsCount = function (_ref2) {
      var firstVisibleIndex = _ref2.firstVisibleIndex,
          maxWidth = _ref2.maxWidth,
          totalItems = _ref2.totalItems,
          itemWidth = _ref2.itemWidth,
          controlWidth = _ref2.controlWidth,
          itemMargin = _ref2.itemMargin;

      var availableWidth = void 0;
      var visibleItemsCount = void 0;

      /**
       * Get the number of visible items which can be shown at a time.
       *
       * @param {number} fullWidth Total available width in which items are to be shown
       * @param {number} elementWidth Width of each individual item to be inserted
       * @param {number} elementMargin Margin between two items
       * @returns {number}
       */
      // Keeping this function inside because
      // a) it's only need to be used internally in this function
      // b) All names I could come up for it can confuse user for this.getVisibleItemsCount
      var calculateVisibleItemsCount = function calculateVisibleItemsCount(fullWidth, elementWidth, elementMargin) {
        return Math.floor(fullWidth / (elementWidth + elementMargin));
      };

      availableWidth = maxWidth;
      var prevButtonVisible = _this.isPrevButtonVisible(firstVisibleIndex);

      if (prevButtonVisible) {
        availableWidth -= controlWidth + itemMargin;
      }

      visibleItemsCount = calculateVisibleItemsCount(availableWidth, itemWidth, itemMargin);
      var nextButtonVisible = _this.isNextButtonVisible(firstVisibleIndex, totalItems, visibleItemsCount);

      if (nextButtonVisible) {
        availableWidth -= controlWidth + itemMargin;
        visibleItemsCount = calculateVisibleItemsCount(availableWidth, itemWidth, itemMargin);
      }

      return visibleItemsCount;
    };

    _this.handleScrollToDirection = function (direction, visibleItemsCount) {
      return function () {
        var index = _this.getIndexToScrollTo({
          direction: direction,
          firstVisibleIndex: _this.props.firstVisibleIndex,
          scrollStepDistance: _this.props.scrollStepDistance,
          totalItems: _this.props.items.length,
          visibleItemsCount: visibleItemsCount
        });

        _this.props.onItemsScroll(index);
      };
    };

    _this.handleActivateItem = function (item) {
      _this.props.onItemActivate(item.key);
    };

    _this.getItemJsx = function (visibleItemsCount) {
      /**
       * Creates JSX for one item
       *
       * @param {JSX} item Valid JSX for the item's content
       * @param {number} index Index of item in the items array
       * @param {number} visibleItemsCount Total number of visible items
       *
       * @returns {JSX}
       */
      return function (item, index) {
        var self = _this;
        var visibleItemIndeces = _lodash2.default.range(_this.props.firstVisibleIndex, visibleItemsCount + _this.props.firstVisibleIndex);
        var isVisible = _lodash2.default.includes(visibleItemIndeces, index);

        /*
         * Here we could return only those items which need be visible, but I am returning all.
         * Since React does optimal DOM operations, rendering all items at once and then updating their
         * style is much less expensive then rendering a set of new list-items everytime.
         */
        return _react2.default.createElement(
          'li',
          {
            className: 'react-carousel__cell' + (_this.props.activeItemId === item.key ? ' react-carousel__cell--active' : ''),
            key: item.key,
            onClick: function onClick() {
              self.handleActivateItem(item);
            },
            style: {
              display: isVisible ? 'list-item' : 'none',
              marginRight: _this.props.itemMargin,
              width: _this.props.itemWidth
            }
          },
          item
        );
      };
    };

    _this.state = { maxWidth: null };
    _this.resizeEventListener = _lodash2.default.debounce(function () {
      _this.setState({ maxWidth: _this.wrapperElement.offsetWidth });
    }, 100);
    return _this;
  }

  _createClass(Carousel, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      this.setState({ maxWidth: this.wrapperElement.offsetWidth });
      window.addEventListener('resize', this.resizeEventListener);
    }
  }, {
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      window.removeEventListener('resize', this.resizeEventListener);
    }

    /**
     * Get the index component should safely scroll to.
     *
     * @param {Object} options
     * @param {number} options.totalItems Total number of items being displayed
     * @param {number} options.currentVisibleIndex Index of item in front
     * @param {string} options.direction next|previous Direction in which to scroll
     * @param {number} options.visibleItemsCount Number of items displayed at one time in component
     * @param {number} options.stepDistance Number of items to scroll in one step
     *
     * @returns {number} Index of item to which to scroll
     */


    /**
     * Tells if the next button shall be visible or not.
     */


    /**
     * Tells if the previous control button shall be visible or not.
     */


    /**
     * Get number of items that can be shown at a point in time.
     *
     * @param {Object} options Arguments in an object because they are too many
     * @property {number} firstVisibleIndex Index to which the carousel is scrolled to i.e first visible index
     * @property {number} maxWidth Maximum width available to the carousel
     * @property {number} totalItems Total number of items to be shown
     * @property {number} itemWidth Width of each item
     * @property {number} controlWidth Width of each control button
     * @property {number} itemMargin Margin between items
     * @returns {number}
     */


    /**
     * Helper function to avoid typing so many variables twice for two control buttons since most of the variables are already in scope.
     *
     * @param {string} direction direction in which to scroll
     * @param {number} visibleItemsCount Total number of visible items
     * @returns {undefined}
     */


    /**
     * Creates functoin that creates JSX for one item
     *
     * @param {number} visibleItemsCount Total number of visible items
     *
     * @returns {Function}
     */

  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props = this.props,
          controlWidth = _props.controlWidth,
          firstVisibleIndex = _props.firstVisibleIndex,
          itemMargin = _props.itemMargin,
          items = _props.items,
          itemWidth = _props.itemWidth,
          maxWidth = _props.maxWidth;

      var totalItems = items.length;
      var prevButtonVisible = this.isPrevButtonVisible(firstVisibleIndex);
      var visibleItemsCount = this.getVisibleItemsCount({
        controlWidth: controlWidth,
        firstVisibleIndex: firstVisibleIndex,
        itemMargin: itemMargin,
        itemWidth: itemWidth,
        maxWidth: maxWidth,
        totalItems: totalItems
      });
      var nextButtonVisible = this.isNextButtonVisible(firstVisibleIndex, totalItems, visibleItemsCount);

      return _react2.default.createElement(
        'div',
        {
          className: 'react-carousel', ref: function ref(element) {
            _this2.wrapperElement = element;
          } },
        _react2.default.createElement(
          'ul',
          { className: 'clearfix react-carousel__body' },
          _react2.default.createElement(
            'li',
            {
              className: 'react-carousel__control-cell carousel__control-cell--previous clearfix',
              onClick: this.handleScrollToDirection('previous', visibleItemsCount),
              style: {
                display: prevButtonVisible ? 'list-item' : 'none',
                marginRight: itemMargin,
                width: controlWidth
              }
            },
            _react2.default.createElement('span', { className: 'react-carousel__control-cell__icon--previous' })
          ),
          _lodash2.default.map(items, this.getItemJsx(visibleItemsCount)),
          _react2.default.createElement(
            'li',
            {
              className: 'react-carousel__control-cell react-carousel__control-cell--next clearfix',
              onClick: this.handleScrollToDirection('next', visibleItemsCount),
              style: {
                display: nextButtonVisible ? 'list-item' : 'none',
                width: controlWidth
              }
            },
            _react2.default.createElement('span', { className: 'react-carousel__control-cell__icon--next' })
          )
        )
      );
    }
  }]);

  return Carousel;
}(_react.Component);

Carousel.propTypes = {
  activeItemId: _react.PropTypes.string,
  controlWidth: _react.PropTypes.number,
  firstVisibleIndex: _react.PropTypes.number,
  itemMargin: _react.PropTypes.number,
  items: _react.PropTypes.arrayOf(_react.PropTypes.object).isRequired,
  itemWidth: _react.PropTypes.number,
  onItemActivate: _react.PropTypes.func,
  onItemsScroll: _react.PropTypes.func,
  scrollStepDistance: _react.PropTypes.number
};
Carousel.defaultProps = {
  controlWidth: 30,
  firstVisibleIndex: 0,
  itemMargin: 1,
  itemWidth: 50,
  scrollStepDistance: 3
};
exports.default = Carousel;
//# sourceMappingURL=index.js.map