/* global describe, it, global, require, context */
/* eslint-disable max-nested-callbacks */

import chai, {
  expect
} from 'chai';
import sinonChai from 'sinon-chai';
import CarouselComponent from '../src/';

chai.use(sinonChai);

describe('Carousel', () => {
  const Carousel = new CarouselComponent();

  describe('.getIndexToScrollTo', () => {
    it('never give index larger than total items', () => {
      const act1 = Carousel.getIndexToScrollTo({
        direction: 'next',
        firstVisibleIndex: 0,
        scrollStepDistance: 9,
        totalItems: 10,
        visibleItemsCount: 4
      });

      const act2 = Carousel.getIndexToScrollTo({
        direction: 'next',
        firstVisibleIndex: 6,
        scrollStepDistance: 6,
        totalItems: 10,
        visibleItemsCount: 5
      });

      expect(act1).to.be.below(11);
      expect(act2).to.not.be.above(10);
    });

    it('never give index less than 0', () => {
      const act1 = Carousel.getIndexToScrollTo({
        direction: 'previous',
        firstVisibleIndex: 0,
        scrollStepDistance: 9,
        totalItems: 10,
        visibleItemsCount: 4
      });
      const act2 = Carousel.getIndexToScrollTo({
        direction: 'previous',
        firstVisibleIndex: 4,
        scrollStepDistance: 6,
        totalItems: 10,
        visibleItemsCount: 4
      });

      expect(act1).to.not.be.below(0);
      expect(act2).to.not.be.below(0);
    });
  });

  describe('.isNextButtonVisible', () => {
    context('all items are visible', () => {
      it('returns false', () => {
        const firstVisibleIndex = 0;
        const totalItems = 10;
        const visibleItemsCount = 10;
        const isNextButtonVisible = Carousel.isNextButtonVisible(firstVisibleIndex, totalItems, visibleItemsCount);

        expect(isNextButtonVisible).to.equal(false);
      });
    });

    context('no more items to display', () => {
      it('returns false', () => {
        const firstVisibleIndex = 6;
        const totalItems = 10;
        const visibleItemsCount = 4;
        const isNextButtonVisible = Carousel.isNextButtonVisible(firstVisibleIndex, totalItems, visibleItemsCount);

        expect(isNextButtonVisible).to.equal(false);
      });
    });

    context('there are items to display', () => {
      it('returns true', () => {
        const firstVisibleIndex = 5;
        const totalItems = 10;
        const visibleItemsCount = 4;
        const isNextButtonVisible = Carousel.isNextButtonVisible(firstVisibleIndex, totalItems, visibleItemsCount);

        expect(isNextButtonVisible).to.equal(true);
      });
    });
  });

  describe('.isPrevButtonVisible', () => {
    it('returns false if carousel isn\'t scrolled', () => {
      const isPrevButtonVisible = Carousel.isPrevButtonVisible(0);

      expect(isPrevButtonVisible).to.equal(false);
    });

    it('returns true if crousel is scrolled', () => {
      const isPrevButtonVisible = Carousel.isPrevButtonVisible(4);

      expect(isPrevButtonVisible).to.equal(true);
    });
  });

  describe('getVisibleItemsCount', () => {
    it('returns correct results', () => {
      const expectedWithNoMargin = (500 - 20) / 40;
      const expectedWithMargin = Math.floor((500 - 22) / 41);
      const countWithNoMargin = Carousel.getVisibleItemsCount({
        controlWidth: 10,
        firstVisibleIndex: 4,
        itemMargin: 0,
        itemWidth: 40,
        maxWidth: 500,
        totalItems: 10
      });
      const countWithMargin = Carousel.getVisibleItemsCount({
        controlWidth: 10,
        firstVisibleIndex: 4,
        itemMargin: 1,
        itemWidth: 40,
        maxWidth: 500,
        totalItems: 10
      });

      expect(countWithNoMargin).to.be.equal(expectedWithNoMargin);
      expect(countWithMargin).to.be.equal(expectedWithMargin);
    });
  });
});
